From: Greg KH <gregkh@suse.de>
To: linux-kernel@vger.kernel.org, stable@kernel.org
Cc: stable-review@kernel.org, torvalds@linux-foundation.org,
akpm@linux-foundation.org, alan@lxorguk.ukuu.org.uk,
Yinghai Lu <yinghai@kernel.org>,
Stefano Stabellini <stefano.stabellini@eu.citrix.com>,
"H. Peter Anvin" <hpa@zytor.com>
Subject: [51/63] x86: Cleanup highmap after brk is concluded
Date: Fri, 25 Mar 2011 16:44:27 -0700 [thread overview]
Message-ID: <20110325234513.208887551@clark.kroah.org> (raw)
In-Reply-To: <20110325234526.GA18017@kroah.com>
2.6.38-stable review patch. If anyone has any objections, please let us know.
------------------
From: Yinghai Lu <yinghai@kernel.org>
commit e5f15b45ddf3afa2bbbb10c7ea34fb32b6de0a0e upstream.
Now cleanup_highmap actually is in two steps: one is early in head64.c
and only clears above _end; a second one is in init_memory_mapping() and
tries to clean from _brk_end to _end.
It should check if those boundaries are PMD_SIZE aligned but currently
does not.
Also init_memory_mapping() is called several times for numa or memory
hotplug, so we really should not handle initial kernel mappings there.
This patch moves cleanup_highmap() down after _brk_end is settled so
we can do everything in one step.
Also we honor max_pfn_mapped in the implementation of cleanup_highmap.
Signed-off-by: Yinghai Lu <yinghai@kernel.org>
Signed-off-by: Stefano Stabellini <stefano.stabellini@eu.citrix.com>
LKML-Reference: <alpine.DEB.2.00.1103171739050.3382@kaball-desktop>
Signed-off-by: H. Peter Anvin <hpa@zytor.com>
Signed-off-by: Greg Kroah-Hartman <gregkh@suse.de>
---
arch/x86/kernel/head64.c | 3 ---
arch/x86/kernel/setup.c | 5 +++++
arch/x86/mm/init.c | 19 -------------------
arch/x86/mm/init_64.c | 11 ++++++-----
4 files changed, 11 insertions(+), 27 deletions(-)
--- a/arch/x86/kernel/head64.c
+++ b/arch/x86/kernel/head64.c
@@ -77,9 +77,6 @@ void __init x86_64_start_kernel(char * r
/* Make NULL pointers segfault */
zap_identity_mappings();
- /* Cleanup the over mapped high alias */
- cleanup_highmap();
-
max_pfn_mapped = KERNEL_IMAGE_SIZE >> PAGE_SHIFT;
for (i = 0; i < NUM_EXCEPTION_VECTORS; i++) {
--- a/arch/x86/kernel/setup.c
+++ b/arch/x86/kernel/setup.c
@@ -297,6 +297,9 @@ static void __init init_gbpages(void)
static inline void init_gbpages(void)
{
}
+static void __init cleanup_highmap(void)
+{
+}
#endif
static void __init reserve_brk(void)
@@ -922,6 +925,8 @@ void __init setup_arch(char **cmdline_p)
*/
reserve_brk();
+ cleanup_highmap();
+
memblock.current_limit = get_max_mapped();
memblock_x86_fill();
--- a/arch/x86/mm/init.c
+++ b/arch/x86/mm/init.c
@@ -279,25 +279,6 @@ unsigned long __init_refok init_memory_m
load_cr3(swapper_pg_dir);
#endif
-#ifdef CONFIG_X86_64
- if (!after_bootmem && !start) {
- pud_t *pud;
- pmd_t *pmd;
-
- mmu_cr4_features = read_cr4();
-
- /*
- * _brk_end cannot change anymore, but it and _end may be
- * located on different 2M pages. cleanup_highmap(), however,
- * can only consider _end when it runs, so destroy any
- * mappings beyond _brk_end here.
- */
- pud = pud_offset(pgd_offset_k(_brk_end), _brk_end);
- pmd = pmd_offset(pud, _brk_end - 1);
- while (++pmd <= pmd_offset(pud, (unsigned long)_end - 1))
- pmd_clear(pmd);
- }
-#endif
__flush_tlb_all();
if (!after_bootmem && e820_table_end > e820_table_start)
--- a/arch/x86/mm/init_64.c
+++ b/arch/x86/mm/init_64.c
@@ -51,6 +51,7 @@
#include <asm/numa.h>
#include <asm/cacheflush.h>
#include <asm/init.h>
+#include <asm/setup.h>
static int __init parse_direct_gbpages_off(char *arg)
{
@@ -293,18 +294,18 @@ void __init init_extra_mapping_uc(unsign
* to the compile time generated pmds. This results in invalid pmds up
* to the point where we hit the physaddr 0 mapping.
*
* We limit the mappings to the region from _text to _end. _end is
* rounded up to the 2MB boundary. This catches the invalid pmds as
* well, as they are located before _text:
*/
void __init cleanup_highmap(void)
{
unsigned long vaddr = __START_KERNEL_map;
- unsigned long end = roundup((unsigned long)_end, PMD_SIZE) - 1;
+ unsigned long vaddr_end = __START_KERNEL_map + (max_pfn_mapped << PAGE_SHIFT);
+ unsigned long end = roundup((unsigned long)_brk_end, PMD_SIZE) - 1;
pmd_t *pmd = level2_kernel_pgt;
- pmd_t *last_pmd = pmd + PTRS_PER_PMD;
- for (; pmd < last_pmd; pmd++, vaddr += PMD_SIZE) {
+ for (; vaddr + PMD_SIZE - 1 < vaddr_end; pmd++, vaddr += PMD_SIZE) {
if (pmd_none(*pmd))
continue;
if (vaddr < (unsigned long) _text || vaddr > end)
next prev parent reply other threads:[~2011-03-26 0:04 UTC|newest]
Thread overview: 64+ messages / expand[flat|nested] mbox.gz Atom feed top
2011-03-25 23:45 [00/63] 2.6.38.2-stable review Greg KH
2011-03-25 23:43 ` [01/63] ALSA: HDA: Realtek: Avoid unnecessary volume control index on Surround/Side Greg KH
2011-03-25 23:43 ` [02/63] ALSA: hda - VIA: Fix stereo mixer recording no sound issue Greg KH
2011-03-25 23:43 ` [03/63] ALSA: hda - VIA: Fix independent headphone " Greg KH
2011-03-25 23:43 ` [04/63] ALSA: hda - VIA: Add missing support for VT1718S in A-A path Greg KH
2011-03-25 23:43 ` [05/63] ALSA: hda - VIA: Fix invalid A-A path volume adjust issue Greg KH
2011-03-25 23:43 ` [06/63] ALSA: hda - VIA: Fix codec type for VT1708BCE at the right timing Greg KH
2011-03-25 23:43 ` [07/63] ALSA: hda - VIA: Fix VT1708 cant build up Headphone control issue Greg KH
2011-03-25 23:43 ` [08/63] ASoC: mini2440: Fix uda134x codec problem Greg KH
2011-03-25 23:43 ` [09/63] ethtool: Compat handling for struct ethtool_rxnfc Greg KH
2011-03-25 23:43 ` [10/63] Revert "slab: Fix missing DEBUG_SLAB last user" Greg KH
2011-03-25 23:43 ` [11/63] aio: wake all waiters when destroying ctx Greg KH
2011-03-25 23:43 ` [12/63] cgroups: if you list_empty() a head then dont list_del() it Greg KH
2011-03-25 23:43 ` [13/63] shmem: let shared anonymous be nonlinear again Greg KH
2011-03-25 23:43 ` [14/63] mm: swap: unlock swapfile inode mutex before closing file on bad swapfiles Greg KH
2011-03-25 23:43 ` [15/63] mm: compaction: prevent kswapd compacting memory to reduce CPU usage Greg KH
2011-03-25 23:43 ` [16/63] oom: prevent unnecessary oom kills or kernel panics Greg KH
2011-03-25 23:43 ` [17/63] oom: skip zombies when iterating tasklist Greg KH
2011-03-25 23:43 ` [18/63] oom: avoid deferring oom killer if exiting task is being traced Greg KH
2011-03-25 23:43 ` [19/63] PCI hotplug: acpiphp: set current_state to D0 in register_slot Greg KH
2011-03-25 23:43 ` [20/63] Input: xen-kbdfront - advertise either absolute or relative coordinates Greg KH
2011-03-25 23:43 ` [21/63] xen: set max_pfn_mapped to the last pfn mapped Greg KH
2011-03-25 23:43 ` [22/63] Prevent rt_sigqueueinfo and rt_tgsigqueueinfo from spoofing the signal code Greg KH
2011-03-25 23:43 ` [23/63] ALSA: HDA: Fix internal mic on Dell E5420/E5520 Greg KH
2011-03-25 23:44 ` [24/63] mm: PageBuddy and mapcount robustness Greg KH
2011-03-25 23:44 ` [25/63] ext3: skip orphan cleanup on rocompat fs Greg KH
2011-03-25 23:44 ` [26/63] x86: Fix binutils-2.21 symbol related build failures Greg KH
2011-03-25 23:44 ` [27/63] sysctl: restrict write access to dmesg_restrict Greg KH
2011-03-25 23:44 ` [28/63] procfs: fix /proc/<pid>/maps heap check Greg KH
2011-03-25 23:44 ` [29/63] proc: protect mm start_code/end_code in /proc/pid/stat Greg KH
2011-03-25 23:44 ` [30/63] fbcon: Bugfix soft cursor detection in Tile Blitting Greg KH
2011-03-25 23:44 ` [31/63] nfsd41: modify the members value of nfsd4_op_flags Greg KH
2011-03-25 23:44 ` [32/63] nfsd4: minor nfs4state.c reshuffling Greg KH
2011-03-25 23:44 ` [33/63] nfsd4: fix struct file leak Greg KH
2011-03-25 23:44 ` [34/63] nfsd: wrong index used in inner loop Greg KH
2011-03-25 23:44 ` [35/63] [media] uvcvideo: Fix uvc_fixup_video_ctrl() format search Greg KH
2011-03-25 23:44 ` [36/63] [media] uvcvideo: Fix descriptor parsing for video output devices Greg KH
2011-03-25 23:44 ` [37/63] sh: Fix ptrace fpu state initialisation Greg KH
2011-03-25 23:44 ` [38/63] sh: Fix ptrace hw_breakpoint handling Greg KH
2011-03-25 23:44 ` [39/63] USB: Do not pass negative length to snoop_urb() Greg KH
2011-03-25 23:44 ` [40/63] ehci-hcd: Bug fix: dont set a QHs Halt bit Greg KH
2011-03-25 23:44 ` [41/63] usb: musb: blackfin: fix typo in platform driver name Greg KH
2011-03-25 23:44 ` [42/63] usb: musb: blackfin: fix typo in new dev_pm_ops struct Greg KH
2011-03-25 23:44 ` [43/63] usb: musb: blackfin: fix typo in new bfin_musb_vbus_status func Greg KH
2011-03-25 23:44 ` [44/63] USB: uss720 fixup refcount position Greg KH
2011-03-25 23:44 ` [45/63] USB: Fix bad dma problem on WDM device disconnect Greg KH
2011-03-25 23:44 ` [46/63] USB: cdc-acm: fix memory corruption / panic Greg KH
2011-03-25 23:44 ` [47/63] USB: cdc-acm: fix potential null-pointer dereference Greg KH
2011-03-25 23:44 ` [48/63] USB: cdc-acm: fix potential null-pointer dereference on disconnect Greg KH
2011-03-25 23:44 ` [49/63] fix deadlock in pivot_root() Greg KH
2011-03-25 23:44 ` [50/63] fs: assign sb->s_bdi to default_backing_dev_info if the bdi is going away Greg KH
2011-03-25 23:44 ` Greg KH [this message]
2011-03-25 23:44 ` [52/63] drm: check for modesetting on modeset ioctls Greg KH
2011-03-25 23:44 ` [53/63] drm/i915: Disable pagefaults along execbuffer relocation fast path Greg KH
2011-03-25 23:44 ` [54/63] drm/i915: Prevent racy removal of request from client list Greg KH
2011-03-25 23:44 ` [55/63] drm: Fix use-after-free in drm_gem_vm_close() Greg KH
2011-03-25 23:44 ` [56/63] drm/radeon/kms: prefer legacy pll algo for tv-out Greg KH
2011-03-25 23:44 ` [57/63] drm/radeon/kms: fix hardcoded EDID handling Greg KH
2011-03-25 23:44 ` [58/63] perf: Fix tear-down of inherited group events Greg KH
2011-03-25 23:44 ` [59/63] NFS: Fix a hang/infinite loop in nfs_wb_page() Greg KH
2011-03-25 23:44 ` [60/63] SUNRPC: Never reuse the socket port after an xs_close() Greg KH
2011-03-25 23:44 ` [61/63] fs: call security_d_instantiate in d_obtain_alias V2 Greg KH
2011-03-25 23:44 ` [62/63] dcdbas: force SMI to happen when expected Greg KH
2011-03-25 23:44 ` [63/63] ext4: skip orphan cleanup if fs has unknown ROCOMPAT features Greg KH
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20110325234513.208887551@clark.kroah.org \
--to=gregkh@suse.de \
--cc=akpm@linux-foundation.org \
--cc=alan@lxorguk.ukuu.org.uk \
--cc=hpa@zytor.com \
--cc=linux-kernel@vger.kernel.org \
--cc=stable-review@kernel.org \
--cc=stable@kernel.org \
--cc=stefano.stabellini@eu.citrix.com \
--cc=torvalds@linux-foundation.org \
--cc=yinghai@kernel.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -293,18 +294,18 @@ void __init init_extra_mapping_uc(unsign
* to the point where we hit the physaddr 0 mapping.
*
- * We limit the mappings to the region from _text to _end. _end is
- * rounded up to the 2MB boundary. This catches the invalid pmds as
+ * We limit the mappings to the region from _text to _brk_end. _brk_end
+ * is rounded up to the 2MB boundary. This catches the invalid pmds as
* well, as they are located before _text:
*/
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
all inboxes | Powered by JetHome®